@@ -23,6 +23,7 @@
 #include <libgnomeui/gnome-uidefs.h>
 #include <sigc++/convert.h>
 
+
 namespace Gnome {
 namespace UI {
 /*******************************************************************/
@@ -33,6 +34,8 @@ gchar* cpp_strdup(const string& s)
     return c;
   }
 
+
+
 Info_::Info_()
   : callback_(), subtree_()
   {
@@ -156,11 +159,17 @@ void Info_::set_callback(const Info::CallbackW& callback)
   }
 
 
-void Info_::set_subtree(const Array& subtree)
+void Info_::set_subtree(const Array<Info>& subtree)
   {
     subtree_=subtree;
   }
 
+Array<Info>& Info_::get_subtree()
+  {
+    return subtree_; //Removes Begin and End.
+  }
+
+
 /*******************************************************************/
 Info::~Info()
   {
@@ -198,6 +207,11 @@ Info& Info::operator =(const Info& i)
     return *this;
   }
 
+Gtk::Widget* Info::get_widget() const
+{
+  return Gtk::wrap(GnomeUIInfo::widget);
+}
+
 void Info::set_icon(const Icon& icon_)
   {
     pixmap_type=icon_.pixmap_type;
@@ -250,7 +264,7 @@ void Info::init_cbw(Type type_,const Icon& icon_,
 
 
 void Info::init_sub(Type type_,const Icon& icon_,
-            const string& label_,const Array& array,
+            const string& label_,const Array<Info>& array,
             const string& hint_)
   {
     GnomeUIInfo::type=GnomeUIInfoType(type_);
@@ -263,6 +277,8 @@ void Info::init_sub(Type type_,const Icon& icon_,
     info_->connect(*this);
   }
 
+
+
 extern "C" {
 static void
 gnomemm_info_connect (GnomeUIInfo *uiinfo, gchar *signal_name,
@@ -291,6 +307,34 @@ Separator::operator Gtk::Menu_Helpers::Element()
 
 /*******************************************************************/
 
+SubTree::SubTree()
+  {
+  }
+
+
+SubTree::SubTree(const string& label,const Array<Info>& uitree,
+                 const string& tip /* =string() */)
+  {
+    init_sub(SUBTREE, Icon(), label, uitree, tip);
+  }
+
+SubTree::SubTree(const Icon& icon, const string& label,
+                 const Array<Info>& uitree, const string& tip /* =string() */)
+  {
+    init_sub(SUBTREE, icon, label, uitree, tip);
+  }
+
+SubTree::~SubTree()
+  {
+  }
+
+
+Array<Info>& SubTree::get_uitree()
+{
+  Info_* pInfo_ = info_();
+  return pInfo_->get_subtree();
+}
+
 extern "C" {
 
 static void 
@@ -327,7 +371,7 @@ GnomeUIBuilderData RadioTree::build_data_ =
      &gnomemm_radio_info_connect, 0, 0, 0, 0
   };
 
-RadioTree::RadioTree(const Gnome::UI::Array& array)
+RadioTree::RadioTree(const Gnome::UI::Array<Info>& array)
   {
     GnomeUIInfo::type=GnomeUIInfoType(RADIOITEMS);
     Gnome::UI::Info_ *info_=new Gnome::UI::Info_();
@@ -399,7 +443,7 @@ void ConfigureItem::init(const CallbackW &cv, GnomeUIInfoConfigurableTypes ct, c
 
 namespace Menus {
  
-New::New(const UI::Array& tree)
+New::New(const UI::Array<Info>& tree)
   : UI::Menu("_New",tree)
   {
     set_icon(UI::Icon(GNOME_STOCK_MENU_NEW));
@@ -423,8 +467,8 @@ namespace UI {
 namespace {
 struct UIArrayHolder
   {
-     Gnome::UI::Array info_;
-     UIArrayHolder(const UI::Array& info): info_(info) {}
+     Gnome::UI::Array<Info> info_;
+     UIArrayHolder(const UI::Array<Info>& info): info_(info) {}
      static void destroy(void* d)
        {
          delete ((UIArrayHolder*)d);
@@ -434,7 +478,7 @@ struct UIArrayHolder
 
 
 void fill (Gtk::MenuShell& menu_shell,
-           const UI::Array& info,
+           const UI::Array<Info>& info,
            Gtk::AccelGroup& accel_group,
            gboolean         uline_accels,
            gint             pos)
@@ -444,7 +488,7 @@ void fill (Gtk::MenuShell& menu_shell,
 }
 
 void fill (Gtk::Toolbar& toolbar, 
-           const UI::Array &info, 
+           const UI::Array<Info> &info,
            Gtk::AccelGroup& accel_group)
 {
   toolbar.set_data_full("gnomemm-uihold",new UIArrayHolder(info),UIArrayHolder::destroy);
