@@ -27,6 +27,7 @@
 #include <gtk--/accelgroup.h>
 #include <libgnomeui/gnome-app.h>
 #include <libgnomeui/gnome-app-helper.h>
+#include <vector>
 
 namespace Gnome {
 namespace UI {
@@ -36,14 +37,17 @@ class Icon;
 // Info is the base of the UI item tree it represents broad types
 // of GUI items for construction later.
 
+template<class T_Info>
 class Array;
+
 class Info : protected GnomeUIInfo
 {
   /* Note when deriving this, you must not add any fields nor may you
      add any virtuals */
 
   friend class Info_;
-  friend class Array;
+  friend class Array<Info>;
+
   // must not be dynamic
   void* operator new(size_t s);
 public:
@@ -71,6 +75,8 @@ public:
   ~Info();
   Info& operator =(const Info&);
 
+  Gtk::Widget* Info::get_widget() const;
+
   Type type() const
     { return Type(GnomeUIInfo::type); }
   void set_icon(const Icon& icon);
@@ -85,189 +91,12 @@ protected:
             const string& str,const CallbackW& cb,
             const string& tooltip);
   void init_sub(Type type_,const Icon& icon,
-            const string& str,const Array& sub,
+            const string& str,const Array<Info>& sub,
             const string& tooltip);
   Info_* info_()
-   { return static_cast<Info_*>(unused_data); }
+   { return static_cast<Info_*>(GnomeUIInfo::unused_data); }
 };
 
-// begin marker for C structures (not really for user use)
-class Begin : public Info
-{
-static GnomeUIBuilderData build_data_;
-public:
-  Begin() { init(BUILDER); GnomeUIInfo::moreinfo=&build_data_; }
-};
-
-// end marker for C structures (not really for user use)
-class End : public Info
-{
-public:
-  End() { init(END); }
-};
-
-namespace Array_Helpers
-{
-template<class T>
-struct Traits
-  {
-    typedef typename T::const_iterator iterator;
-    static iterator begin(const T& t) {return t.begin();}
-    static iterator end(const T& t) {return t.end();}
-  };
-
-// You must place an End() to use this type
-struct Traits<Info*>
-  {
-    typedef const Info* iterator;
-    static iterator begin(const Info* t) {return t;}
-    static iterator end(const Info* t) {return t+64;}
-  };
-
-template <size_t N>
-struct Traits<Info[N]> 
-  {
-    typedef const Info* iterator;
-    static iterator begin(const Info* t) {return t;}
-    static iterator end(const Info* t) {return &t[N];}
-  };
-
-} /* namespace Array_ */
-
-
-// Array converter class
-class Array 
-{
-  void* operator new (size_t s);  // not implemented
-  Info* data_;
-  Info* begin_;
-  int size_;
-
-  template <class I> void create(I b, I e);
-public:
-  typedef Info value_type;
-  typedef Info& reference_type;
-  typedef Info* const iterator;
-  typedef Info* const const_iterator;
-  typedef Info* const pointer;
-  typedef Info* const const_pointer;
-  typedef size_t size_type;
-  typedef ptrdiff_t difference_type;
-
-  Array& operator=(const Array& a)
-    { 
-      if (this==&a) return *this;
-      delete [] data_; data_=0; size_=0; 
-      create(a.begin(),a.end());
-      return *this;
-    }
-  Array()
-    : data_(0),begin_(0),size_(0)
-    { create((Info*)0,(Info*)0); }
-  Array(Array& a)
-    : data_(0),begin_(0),size_(0)
-    { create(a.begin(),a.end()); }
-  template <class T>
-  Array(const T& t)
-    : data_(0),begin_(0),size_(0)
-    { create(Array_Helpers::Traits<T>::begin(t),
-             Array_Helpers::Traits<T>::end(t)); } 
-  template <class I>
-  Array(I b, I e)
-    : data_(0),begin_(0),size_(0)
-    { create(b, e); }
-
-  ~Array() { delete [] data_; data_=0; size_=0; }
-  
-  size_t size() const { return size_; }
-
-  iterator begin() const
-    { return begin_; }
-  iterator end() const
-    { return begin_ + size_; }
-  reference_type operator[](size_t n) const
-    { return begin_[n]; }
-
-  GnomeUIInfo* gtkobj() const 
-    { return data_; }
-};
-
-template <class I> 
-void Array::create(I b, I e)
-  {
-    // NULL list 
-    if (b==e)
-      {
-        data_=new End[1];
-        return;
-      }
-  
-    // determine number of Items
-    for (I b2=b;b2!=e;++b2,++size_)
-      if (b2->type()==Info::END) break;
-
-    // must have a builder data on head
-    if (b->type()!=Info::BUILDER) 
-      {
-        begin_=data_=new Info[size_+2];
-        new (begin_) Begin();
-        begin_++;
-      }    
-    else 
-      begin_=data_=new Info[size_+1];
-
-    // Copy data
-    for (int i=0;b!=e;++b,++i)
-      new (&begin_[i]) Info(*b);
-    new (&begin_[size_]) End();
-  }
-
-// This is a refcounted holder to deal with C interface badness
-// users don't need to deal with these unless they are making new Info types.
-class Info_
-{
-  int ref_count_;
-  Info_(const Info_&);
-
-protected:
-  virtual ~Info_();
-
-public:
-  Info::CallbackW callback_;
-  Array subtree_;
-  gchar* label_;
-  gchar* hint_;
-
-  void ref();
-  void unref();
-
-  virtual void connect(Info&);
-  void set_callback(const Info::Callback&);
-  void set_callback(const Info::CallbackW&);
-  void set_subtree(const Array&);
-
-  Info_();
-  Info_(const string& label, const string& hint);
-};
-
-
-//: Fill a menu with UI::Info items.
-// 'wrapper' for gnome_app_fill_menu()
-void fill (Gtk::MenuShell& menu_shell,
-           const UI::Array& info,
-           Gtk::AccelGroup& accel_group,
-           gboolean       uline_accels = true,
-           gint           pos = 0);
-
-
-//: Fill a toolbar with UI::Info items.
-// 'wrapper' for gnome_app_fill_toolbar()
-void fill (Gtk::Toolbar& toolbar, 
-           const UI::Array &info, 
-           Gtk::AccelGroup& accel_group);
-
-
-
 /*******************************************************/
 // Icons are used to represent a standard Pixmap with various states
 
@@ -310,6 +139,220 @@ struct IconFile : public Icon
 };
 
 
+// subtree or submenu
+class SubTree : public Info
+{
+protected:
+  SubTree();
+public:
+  SubTree(const string& label,const Array<Info>& uitree,
+          const string& tip=string());
+  SubTree(const Icon& icon, const string& label,
+          const Array<Info>& uitree, const string& tip=string());
+  ~SubTree();
+
+  Array<Info>& get_uitree();
+};
+typedef SubTree Menu;
+
+
+
+
+// begin marker for C structures (not really for user use)
+class Begin : public Info
+{
+static GnomeUIBuilderData build_data_;
+public:
+  Begin() { init(BUILDER); GnomeUIInfo::moreinfo=&build_data_; }
+};
+
+// end marker for C structures (not really for user use)
+class End : public Info
+{
+public:
+  End() { init(END); }
+};
+
+namespace Array_Helpers
+{
+
+template<class T>
+struct Traits
+  {
+    typedef typename T::const_iterator iterator;
+    static iterator begin(const T& t) {return t.begin();}
+    static iterator end(const T& t) {return t.end();}
+  };
+
+// You must place an End() to use this type
+template<class T_Info>
+struct Traits<T_Info*>
+  {
+    typedef const T_Info* iterator;
+    static iterator begin(const T_Info* t) {return t;}
+    static iterator end( T_Info* t) {return t+64;} //64?
+  };
+
+template<size_t N, class T_Info>
+struct Traits<T_Info[N]> 
+  {
+    typedef const T_Info* iterator;
+    static iterator begin(const T_Info* t) {return t;}
+    static iterator end(const T_Info* t) {return &t[N];}
+  };
+
+} /* namespace Array_Helpers */
+
+
+// Array converter class
+template<class T_Info>
+class Array 
+{
+  //void* operator new (size_t s);  // not implemented
+  Info* data_;
+  Info* begin_;
+  int size_;
+
+  template <class I> void create(I b, I e);
+public:
+  typedef T_Info value_type;
+  typedef T_Info& reference_type;
+  typedef T_Info* iterator;
+  typedef T_Info* const const_iterator;
+  typedef T_Info* const pointer;
+  typedef T_Info* const const_pointer;
+  typedef size_t size_type;
+  typedef ptrdiff_t difference_type;
+
+  Array& operator=(const Array& a)
+    { 
+      if (this==&a) return *this;
+      delete [] data_; data_=0; size_=0; 
+      create(a.begin(),a.end());
+      return *this;
+    }
+  Array()
+    : data_(0),begin_(0),size_(0)
+    { create((T_Info*)0,(T_Info*)0); }
+  Array(Array& a)
+    : data_(0),begin_(0),size_(0)
+    { create(a.begin(),a.end()); }
+  template <class T>
+  Array(const T& t)
+    : data_(0),begin_(0),size_(0)
+    { create(Array_Helpers::Traits<T>::begin(t),
+             Array_Helpers::Traits<T>::end(t)); } 
+  template <class I>
+  Array(I b, I e)
+    : data_(0),begin_(0),size_(0)
+    { create(b, e); }
+
+  ~Array() { delete [] data_; data_=0; size_=0; }
+  
+  size_t size() const { return size_; }
+
+  iterator begin() const
+    { return static_cast<T_Info*>(begin_); }
+  iterator end() const
+    { return static_cast<T_Info*>(begin_ + size_); }
+  reference_type operator[](size_t n) const
+    { return static_cast<T_Info&>(begin_[n]); }
+
+  GnomeUIInfo* gtkobj() const 
+    { return static_cast<GnomeUIInfo*>(data_); }
+
+  vector<Info> make_return_vector(); //Used by get_uitree().
+  vector<SubTree> make_return_vector_subtrees(); //Used by Gnome::App.
+
+};
+
+
+template <class T_Info>
+template <class I>
+void
+Array<T_Info>::create(I b, I e)
+  {
+    // NULL list
+    if (b==e)
+      {
+        data_=new End[1];
+        return;
+      }
+
+    // determine number of Items
+    for (I b2=b;b2!=e;++b2,++size_)
+      if (b2->type()==Info::END) break;
+
+    // must have a builder data on head
+    if (b->type()!=Info::BUILDER)
+      {
+        begin_=data_=new Info[size_+2]; //plus space for BEGIN and END.
+        new (begin_) Begin(); //constructor without allocation.
+        begin_++; //Hide Begin() element from outside, by keeping it before begin().
+      }
+    else
+      begin_=data_=new Info[size_+1]; //plus space for END.
+
+    // Copy data
+    for (int i=0; b!=e; ++b,++i)
+      new (&begin_[i]) T_Info(*b); //constructor without allocation.
+
+    new (&begin_[size_]) End(); //constructor without allocation.
+
+    //At this point _size excludes the Begin() and End() GNOME internals elements,
+    //so users of begin() and end() will never see them.
+  }
+
+
+// This is a refcounted holder to deal with C interface badness
+// users don't need to deal with these unless they are making new Info types.
+class Info_
+{
+  int ref_count_;
+  Info_(const Info_&); //Prevent use of copy constructor.
+
+protected:
+  virtual ~Info_();
+
+public:
+  Info::CallbackW callback_;
+  Array<Info> subtree_;
+  gchar* label_;
+  gchar* hint_;
+
+  void ref();
+  void unref();
+
+  virtual void connect(Info&);
+  void set_callback(const Info::Callback&);
+  void set_callback(const Info::CallbackW&);
+  void set_subtree(const Array<Info>&);
+  Array<Info>& get_subtree();
+
+  Info_();
+  Info_(const string& label, const string& hint);
+};
+
+
+//: Fill a menu with UI::Info items.
+// 'wrapper' for gnome_app_fill_menu()
+void fill (Gtk::MenuShell& menu_shell,
+           const UI::Array<Info>& info,
+           Gtk::AccelGroup& accel_group,
+           gboolean       uline_accels = true,
+           gint           pos = 0);
+
+
+//: Fill a toolbar with UI::Info items.
+// 'wrapper' for gnome_app_fill_toolbar()
+void fill (Gtk::Toolbar& toolbar, 
+           const UI::Array<Info> &info,
+           Gtk::AccelGroup& accel_group);
+
+
+
+
+
 /*******************************************************/
 class Separator : public Info
 {
@@ -365,29 +408,13 @@ public:
   ~Help() {}
 };
 
-// subtree or submenu
-class SubTree : public Info
-{
-protected:
-  SubTree() {}
-public:
-  SubTree(const Icon& icon, const string& label,
-      const Array& uitree,const string& tip=string())
-    { init_sub(SUBTREE,icon,label,uitree,tip); }
-  SubTree(const string& label,const Array& uitree,
-      const string& tip=string())
-    { init_sub(SUBTREE,Icon(),label,uitree,tip); }
-  ~SubTree() {}
-};
-typedef SubTree Menu;
-
 
 // this tree can only hold Items.
 class RadioTree : public Gnome::UI::Info
 {
 static GnomeUIBuilderData build_data_;
 public:
-  RadioTree(const Gnome::UI::Array& array);
+  RadioTree(const Gnome::UI::Array<Info>& array);
   ~RadioTree() {}
 };
 
@@ -497,7 +524,7 @@ namespace Menus {
 // New: If you have more than one New type then use this tree: 
 struct New : public UI::Menu
 {
-  New(const UI::Array& tree);
+  New(const UI::Array<Info>& tree);
   ~New() {}
 };
 
@@ -505,7 +532,7 @@ struct New : public UI::Menu
 #define STOCK_MENU(X,Y) \
 struct X : public UI::Menu \
 { \
-  X(const UI::Array& tree = UI::Array()) \
+  X(const UI::Array<Info>& tree = UI::Array<Info>()) \
   : UI::Menu(Y,tree) \
   {} \
   ~X() \
